/**
 * @file dnload.c
 *
 * DnLoad - Download on-board processor code to active cards.
 *
 * Copyright: 2000-2003 Thomas Wintergerst. All rights reserved.
 *
 * $FreeBSD$
 * $Id: dnload.c,v 1.7.2.2 2004/12/23 10:41:44 thomas Exp $
 * Project  CAPI for BSD
 * Target   avmaicctl - Maintenance tool for AVM active ISDN controllers
 * @date    01.01.2000
 * @author  "Thomas Wintergerst" <twinterg@gmx.de>
 * <p>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * </p>
 *
 * This file is heavily derived from diehlctl by Martin Husemann.
 */

#include <sys/cdefs.h>
__FBSDID("$FreeBSD$");

/* system includes */
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <capi20.h>
#include <capi_bsd.h>

/* import includes */

#define __DNLOAD__

/* local includes */
#include "avmaicctl.h"





/* === public definitions ================================================ */





/* === private definitions =============================================== */





/* the D-channel protocols available for AVM active ISDN controllers */
#define DP_ERROR	-1
#define DP_NONE		0
#define DP_DSS1		1
#define DP_D64S		2
#define DP_D64S2	3
#define DP_D64SD	4
#define DP_DS01		5
#define DP_DS02		6
#define DP_CT1		7
#define DP_VN3		8
#define DP_AUSTEL	9
#define DP_5ESS		10	/* need SPID, SPID2, DN, DN2 */
#define DP_NI1		11	/* need SPID, SPID2, DN, DN2 */
#define DP_DSS1MOBIL	12
#define DP_1TR6MOBIL	13
#define DP_GSM		14
#define DP_1TR6		15
#define DP_T1		16



/* mapping between the string and the identifier for the D-channel protocols */
static struct pmap
{
   const char *pszName;
   int         iProtocol;
} g_pmap [] =
   {
      { "DSS1",   DP_DSS1   },
      { "D64S",   DP_D64S   },
      { "D64S2",  DP_D64S2  },
      { "D64SD",  DP_D64SD  },
      { "DS01",   DP_DS01   },
      { "DS02",   DP_DS02   },
      { "CT1",    DP_CT1    },
      { "VN3",    DP_VN3    },
      { "AUSTEL", DP_AUSTEL },
      { "5ESS",   DP_5ESS   },
      { "NI1",    DP_NI1    },
      { "1TR6",   DP_1TR6   },
      { "T1",     DP_T1     },
      { NULL,     DP_NONE   }
   };





/* === prototypes for private functions ================================== */





/* get the D-channel identifier from a string */
static int dchan_protocol
   (const char *pszProtName);

/* add a piece of configuration data to a configuration block */
static void add_patch_value
   (uint8_t    *pabConfigData,
    size_t      nMaxLenConfigData,
    size_t     *pnCurrPos,
    const char *pszName,
    const char *pacValue,
    size_t      nLenValue);

/* create a block of configuration data for a controller */
static size_t create_config_block
   (uint8_t    *pabConfigData,
    size_t      nMaxLenConfigData,
    int         iDChnProt,
    int         fP2P,
    const char *pDn1,
    const char *pSpid1,
    const char *pDn2,
    const char *pSpid2);





/* === definition of public functions ==================================== */





/*
        download firmware and configuration data to a controller
        --------------------------------------------------------
*/

void download
   (unsigned  uController,
    int       argc,
    char     *argv [])
{
   CAPICtlrDriverInfo_t  capiDrvInfo;
   CAPICtlrDataBlock_t   aCtlrDataBlocks [2];
   int                   fdBootCode;
   struct stat           statBuf;
   int                   fSetConfig;
   int                   iDChnProt = DP_DSS1;
   int                   fP2P = 0;
   char                 *pDn1 = NULL;
   char                 *pSpid1 = NULL;
   char                 *pDn2 = NULL;
   char                 *pSpid2 = NULL;
   uint8_t               abConfigData [1024];
   size_t                nLenConfigData = 0;
   unsigned              uRes;

   /* first get driver information about the controller specified */
   uRes = capi20_get_ctlr_driver_info (uController, &capiDrvInfo);
   if (uRes != CAPI_OK)
   {
      fprintf (stderr,
               "%s: error getting driver info about controller %u: 0x%04x\n",
               e_pszProgName, uController, uRes);
      exit (1);
   }
   
   /* check if this is really a AVM active ISDN controller */
   if (strcasecmp (capiDrvInfo.szDriverName, AVMAIC_DRIVER_NAME) != 0)
   {
      fprintf (stderr,
               "%s: controller %u is no AVM active ISDN controller\n",
               e_pszProgName, uController);
      exit (1);
   }

   /* download may only performed for the first port of a multi port board */
   if (C_GET_WORD (capiDrvInfo.wPortIdx) != 0)
   {
      fprintf (stderr,
               "%s: controller %u is not the first port of its board\n",
               e_pszProgName, uController);
      exit (1);
   }
   
   /* the controller specified is valid for download; evaluate further command
    * line arguments
    */

   /* the first argument must be the boot code file */
   if (argc <= 0)
   {
      /* no boot code file --> do only reset controller (i.e. do an unload) */
      uRes = capi20_reset_ctlr (uController, 0, NULL);
      if (uRes != CAPI_OK)
      {
         fprintf (stderr,
                  "%s: error unloading controller %u: 0x%04x\n",
	          e_pszProgName, uController, uRes);
         exit(1);
      }
      printf ("%s: controller %u successfully unloaded\n",
	      e_pszProgName, uController);
      return;
   }
   if (stat (argv [0], &statBuf) != 0)
   {
      perror (argv [0]);
      exit (1);
   }
   if ((statBuf.st_mode & (S_IFREG | S_IREAD)) != (S_IFREG | S_IREAD))
   {
      fprintf (stderr,
               "%s: boot code file \"%s\" not readable or not a regular "
                  "file\n",
	       e_pszProgName, argv [0]);
      exit (1);
   }

   /* if present 2nd argument is the D-channel protocol */
   if (argc >= 2)
   {
      iDChnProt = dchan_protocol (argv [1]);
      if (iDChnProt < 0)
      {
	 fprintf (stderr, "%s: unknown D-channel protocol \"%s\"\n",
		  e_pszProgName, argv [1]);
	 exit (1);
      }
   }

   /* if present 3rd argument defines MP or P2P or starts definition of SPIDs
    * for 5ESS and NI1
    */
   if (argc >= 3)
   {
      if (strcasecmp (argv [2], "MP") == 0)
      {
	 fP2P = 0;
      }
      else if (strcasecmp (argv [2], "P2P") == 0)
      {
	 fP2P = 1;
      }
      else
      {
	 /* SPID definition only valid if D-channel protocols 5ESS or NI1 */
	 if (iDChnProt != DP_5ESS && iDChnProt != DP_NI1)
	 {
	    fprintf (stderr,
                     "%s: SPID configuration only vaild for D-channel "
                        "protocols 5ESS or NI1\n",
		     e_pszProgName);
	    exit (1);
	 }

	 pDn1 = argv [2];
	 pSpid1 = strchr (pDn1, ':');
	 if (! pSpid1)
	 {
	    fprintf (stderr,
                     "%s: DN1 and SPID1 must be separated by ':' in \"%s\"\n",
		     e_pszProgName, argv [2]);
	    exit (1);
	 }
	 *pSpid1++ = '\0';

	 if (argc >= 4)
	 {
	    pDn2 = argv [3];
	    pSpid2 = strchr (pDn2, ':');
	    if (! pSpid2)
	    {
	       fprintf (stderr,
                        "%s: DN2 and SPID2 must be separated by ':' in "
                           "\"%s\"\n",
			e_pszProgName, argv [3]);
	       exit (1);
	    }
	    *pSpid2++ = '\0';
	 }
      }
   }

   /* if necessary create configuration block for a AVM controller */
   /* Note: For D-channel protocol 1TR6 a special firmware file must be used.
    *       It is the responsibility of the caller to use the correct file.
    *       Here we can only check if another than the default protocol for a
    *       firmware file is used. So for DSS1 or 1TR6 with no other changed
    *       settings no configuration data block is created.
    */
   fSetConfig = 0;
#if 0
   if ((iDChnProt != DP_DSS1 && iDChnProt != DP_1TR6)
       || fP2P ||
       (pDn1 && pSpid1) || (pDn2 && pSpid2))
#endif /* 0 */
   {
      nLenConfigData = create_config_block
			  (abConfigData, sizeof (abConfigData),
			   iDChnProt, fP2P,
			   pDn1, pSpid1, pDn2, pSpid2);
      if (nLenConfigData <= 0)
      {
	 exit (1);
      }
      fSetConfig = 1;
   }

   /* open firmware file */
   fdBootCode = open (argv [0], O_RDONLY);
   if (fdBootCode < 0)
   {
      perror (argv [0]);
      exit (1);
   }

   /* prepare the arguments for the download request */
   bzero (aCtlrDataBlocks, sizeof (aCtlrDataBlocks));
   aCtlrDataBlocks [0].nLenDataBlock = statBuf.st_size;
   aCtlrDataBlocks [0].paucDataBlock =
      mmap (NULL, statBuf.st_size, PROT_READ, MAP_PRIVATE, fdBootCode, 0);
   if (aCtlrDataBlocks [0].paucDataBlock == MAP_FAILED)
   {
      perror (argv [0]);
      close (fdBootCode);
      exit (1);
   }
   if (fSetConfig)
   {
      aCtlrDataBlocks [1].nLenDataBlock = nLenConfigData;
      aCtlrDataBlocks [1].paucDataBlock = abConfigData;
   }

   /* now finally perform the download request */
   uRes = capi20_reset_ctlr (uController,
                             fSetConfig ? 2 : 1,
                             aCtlrDataBlocks);
   close (fdBootCode);
   if (uRes != CAPI_OK)
   {
      fprintf (stderr,
               "%s: error downloading firmware to controller %u: 0x%04x\n",
	       e_pszProgName, uController, uRes);
      exit(1);
   }

   printf ("%s: downloading firmware to controller %u successful\n",
	   e_pszProgName, uController);

} /* download */





/* === definition of private functions =================================== */





/*
        get the D-channel identifier from a string
        ------------------------------------------
*/

static int dchan_protocol
   (const char *pszProtName)
{
   struct pmap *p;

   for (p = g_pmap; p->pszName; p++)
   {
      if (strcasecmp (pszProtName, p->pszName) == 0)
      {
	 return (p->iProtocol);
      }
   }

   return (DP_ERROR);
} /* dchan_protocol */





/*
        add a piece of configuration data to a configuration block
        ----------------------------------------------------------
*/

static void add_patch_value
   (uint8_t    *pabConfigData,
    size_t      nMaxLenConfigData,
    size_t     *pnCurrPos,
    const char *pszName,
    const char *pacValue,
    size_t      nLenValue)
{
   size_t nNameLen = strlen (pszName);

   if (*pnCurrPos + nNameLen + nLenValue + 3 > nMaxLenConfigData)
   {
      fprintf (stderr, "%s: unable to add patch value \"%s\", config data buffer too small\n",
	       e_pszProgName, pszName);
      exit (2);
   }

   pabConfigData [(*pnCurrPos)++] = ':';
   memcpy (&(pabConfigData [*pnCurrPos]), pszName, nNameLen);
   *pnCurrPos += nNameLen;
   pabConfigData [(*pnCurrPos)++] = '\0';
   memcpy (&(pabConfigData [*pnCurrPos]), pacValue, nLenValue);
   *pnCurrPos += nLenValue;
   pabConfigData [(*pnCurrPos)++] = '\0';
   pabConfigData [*pnCurrPos + 1] = '\0';
} /* add_patch_value */





/*
        create a block of configuration data for a controller
        -----------------------------------------------------
*/

static size_t create_config_block
   (uint8_t    *pabConfigData,
    size_t      nMaxLenConfigData,
    int         iDChnProt,
    int         fP2P,
    const char *pDn1,
    const char *pSpid1,
    const char *pDn2,
    const char *pSpid2)
{
   size_t nCurrPos = 0;

   if (iDChnProt != DP_T1)
   {
      add_patch_value (pabConfigData, nMaxLenConfigData, &nCurrPos,
		       "WATCHDOG", "1", 1);
      add_patch_value (pabConfigData, nMaxLenConfigData, &nCurrPos,
		       "AutoFrame", "\001", 1);
   }
   else
   {
      add_patch_value (pabConfigData, nMaxLenConfigData, &nCurrPos,
		       "WATCHDOG", "0", 1);
   }

   switch (iDChnProt)
   {
      case DP_T1:
      case DP_NONE:
	 break;

      case DP_DSS1:
	 break;

      case DP_D64S:
      case DP_D64S2:
      case DP_D64SD:
	 fP2P = 0;
	 add_patch_value (pabConfigData, nMaxLenConfigData, &nCurrPos,
			  "FV2", "2", 1);
	 add_patch_value (pabConfigData, nMaxLenConfigData, &nCurrPos,
			  "TEI", "\000", 1);
	 break;

      case DP_DS01:
      case DP_DS02:
	 fP2P = 0;
	 add_patch_value (pabConfigData, nMaxLenConfigData, &nCurrPos,
			  "FV2", "1", 1);
	 add_patch_value (pabConfigData, nMaxLenConfigData, &nCurrPos,
			  "TEI", "\000", 1);
	 break;

      case DP_CT1:
	 add_patch_value (pabConfigData, nMaxLenConfigData, &nCurrPos,
			  "PROTOCOL", "\001", 1);
	 break;

      case DP_VN3:
	 add_patch_value (pabConfigData, nMaxLenConfigData, &nCurrPos,
			  "PROTOCOL", "\002", 1);
	 break;

      case DP_AUSTEL:
	 add_patch_value (pabConfigData, nMaxLenConfigData, &nCurrPos,
			  "PROTOCOL", "\004", 1);
	 break;

      case DP_NI1:
	 fP2P = 0;
	 add_patch_value (pabConfigData, nMaxLenConfigData, &nCurrPos,
			  "PROTOCOL", "\003", 1);
	 if (pDn1 && pSpid1)
	 {
	    add_patch_value (pabConfigData, nMaxLenConfigData, &nCurrPos,
			     "DN", pDn1, strlen (pDn1));
	    add_patch_value (pabConfigData, nMaxLenConfigData, &nCurrPos,
			     "SPID", pSpid1, strlen (pSpid1));
	 }
	 if (pDn2 && pSpid2)
	 {
	    add_patch_value (pabConfigData, nMaxLenConfigData, &nCurrPos,
			     "DN2", pDn2, strlen (pDn2));
	    add_patch_value (pabConfigData, nMaxLenConfigData, &nCurrPos,
			     "SPID2", pSpid2, strlen (pSpid2));
	 }
	 break;

      case DP_5ESS:
	 fP2P = 0;
	 add_patch_value (pabConfigData, nMaxLenConfigData, &nCurrPos,
			  "PROTOCOL", "\005", 1);
	 if (pDn1 && pSpid1)
	 {
	    add_patch_value (pabConfigData, nMaxLenConfigData, &nCurrPos,
			     "DN", pDn1, strlen (pDn1));
	    add_patch_value (pabConfigData, nMaxLenConfigData, &nCurrPos,
			     "SPID", pSpid1, strlen (pSpid1));
	 }
	 if (pDn2 && pSpid2)
	 {
	    add_patch_value (pabConfigData, nMaxLenConfigData, &nCurrPos,
			     "DN2", pDn2, strlen (pDn2));
	    add_patch_value (pabConfigData, nMaxLenConfigData, &nCurrPos,
			     "SPID2", pSpid2, strlen (pSpid2));
	 }
	 break;

      case DP_DSS1MOBIL:
	 add_patch_value (pabConfigData, nMaxLenConfigData, &nCurrPos,
			  "PatchMobileMode", "0", 1);
	 break;

      case DP_1TR6MOBIL:
	 add_patch_value (pabConfigData, nMaxLenConfigData, &nCurrPos,
			  "PatchMobileMode", "0", 1);
	 break;

      case DP_GSM:
	 break;

      case DP_1TR6:
	 break;

      default:
	 return (0);
   }

   if (fP2P)
   {
      add_patch_value (pabConfigData, nMaxLenConfigData, &nCurrPos,
		       "P2P", "\001", 1);
      add_patch_value (pabConfigData, nMaxLenConfigData, &nCurrPos,
		       "TEI", "\000", 1);
   }

   if (nCurrPos > nMaxLenConfigData)
   {
      return (0);
   }

   return (nCurrPos);
} /* create_config_block */
