/**
 * @file main.c
 *
 * Main - Argument parsing and function dispatching.
 *
 * Copyright: 2000-2003 Thomas Wintergerst. All rights reserved.
 *
 * $FreeBSD$
 * $Id: main.c,v 1.9.2.2 2004/12/23 10:41:44 thomas Exp $
 * Project  CAPI for BSD
 * Target   avmaicctl - Maintenance tool for AVM active ISDN controllers
 * @date    01.01.2000
 * @author  "Thomas Wintergerst" <twinterg@gmx.de>
 * <p>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * </p>
 *
 * This file is heavily derived from diehlctl by Martin Husemann.
 */

#include <sys/cdefs.h>
__FBSDID("$FreeBSD$");

/* system includes */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <capi20.h>
#include <capi_bsd.h>

/* import includes */

#define __MAIN__

/* local includes */
#include "avmaicctl.h"





/* === public definitions ================================================ */





/* the name of the program for output */
const char *e_pszProgName = NULL;





/* === private definitions =============================================== */





/* === prototypes for private functions ================================== */





/* print usage information */
static void usage (void);

/* list all controllers installed */
static void listall (void);





/* === definition of public functions ==================================== */





/*
        The main entry point
        ====================
*/

int main
   (int    argc,
    char **argv)
{
   extern char *optarg;
   extern int   optind;
   unsigned     uController = (unsigned) -1;
   int          lflag;
   int          dflag;
   char         ch;

   /* setting up the program name for output */
   e_pszProgName = strrchr (argv [0], '/');
   if (e_pszProgName)
   {
     e_pszProgName++;
   }
   else
   {
     e_pszProgName = argv [0];
   }

   /* parse the command line */
   lflag = dflag = 0;
   while ((ch = getopt (argc, argv, "d:lh?")) != -1)
   {
      switch (ch)
      {
          case 'd':
	     uController = (unsigned) strtoul (optarg, NULL, 0);
	     dflag = 1;
	     break;

          case 'l':
	     lflag = 1;
	     break;

          case '?':
          case 'h':
          default:
	     usage ();
             return (0);
      }
   }

   /* adjust the arguments to be evaluated later if needed */
   argc -= optind;
   argv += optind;

   /* perform requested action */
   if (lflag)
   {
      listall ();
   }
   else if (dflag)
   {
      download (uController, argc, argv);
   }
   else
   {
      usage ();
   }

   return (0);
} /* main */





/* === definition of private functions =================================== */





/*
        print usage information
        -----------------------
*/

static void usage (void)
{
   fprintf (stdout, "\n");
   fprintf (stdout,
            "%s - Download firmware and configuration to active ISDN boards from AVM\n",
            e_pszProgName);
   fprintf (stdout, "\nUsage:\n\n");
   fprintf (stdout, "%s -l\n", e_pszProgName);
   fprintf (stdout, "   list available controllers\n");
   fprintf (stdout, "\n");
   fprintf (stdout,
            "%s -d <controller no.>\n", e_pszProgName);
   fprintf (stdout, "\t[<bootcode file> [<protocol> [MP | P2P |  DN1:SPID1 [DN2:SPID2]]]]\n");
   fprintf (stdout, "   download software and configuration to controller\n");
   fprintf (stdout, "\n");
   fprintf (stdout, "   protocol\tDSS1, D64S, D64S2, D64SD, DS01, DS02,\n");
   fprintf (stdout, "\t\tCT1, VN3, AUSTEL, 5ESS, NI1, 1TR6, T1\n");
   fprintf (stdout,
            "   DN1, DN2, SPID1, SPID2 are only needed for 5ESS and NI1\n");
   fprintf (stdout, "\n");
   fprintf (stdout,
            "   For D-channel protocol 1TR6 there is a special firmware "
               "file.\n");
   fprintf (stdout, "   You are responsible to use the correct firmware.\n");
   fprintf (stdout, "\n");
   fprintf (stdout,
            "   If no boot code file is specified, the controller will only "
               "be reset\n");
   fprintf (stdout, "   (i.e. unloaded).\n");
   fprintf (stdout, "\n");
   
   exit(1);
} /* usage */





/*
        list all controllers installed
        ------------------------------
*/

static void listall (void)
{
   CAPIProfileBuffer_t  profile;
   CAPICtlrDriverInfo_t drvInfo;
   int                  i;
   unsigned             uNumCtlr;
   unsigned             u;
   unsigned             uRes;
   
   /* first get the number of installed controllers */
   uRes = capi20_get_profile (0, &profile);
   if (uRes != CAPI_OK)
   {
      fprintf (stderr,
               "%s: error getting number of installed controllers: 0x%04x\n",
               e_pszProgName, uRes);
      return;
   }
   uNumCtlr = C_GET_WORD (profile.wCtlr);
   if (uNumCtlr == 0)
   {
      printf ("%s: no controllers installed\n", e_pszProgName);
      return;
   }
   
   /* now query all controllers until all found */
   printf ("\n%u controllers installed:\n\n", uNumCtlr);
   for (i = 1, u = 0; i <= 127 && u < uNumCtlr; i++)
   {
      uRes = capi20_get_ctlr_driver_info ((unsigned) i, &drvInfo);
      if (uRes == CRE_CAPI_NOT_INSTALLED)
      {
         /* seems to be a hole in the array of CAPI controllers... */
         continue;
      }
      u++;                              /* count the controller found */
      if (uRes != CAPI_OK)
      {
         fprintf (stderr,
                  "%s: error getting driver info for controller %u: 0x%04x\n",
                  e_pszProgName, (unsigned) i, uRes);
         continue;
      }
      
      printf ("\t%u: name \"%s\", type \"%s\", driver \"%s\", driver unit "
                 "%u\n",
              (unsigned) i, drvInfo.szCtlrName, drvInfo.szCtlrTypeName,
              drvInfo.szDriverName, C_GET_WORD (drvInfo.wDrvUnitNum));
      if (C_GET_WORD (drvInfo.wNumPorts > 1))
      {
         printf ("\t\tport %u of %u\n",
                 C_GET_WORD (drvInfo.wPortIdx),
                 C_GET_WORD (drvInfo.wNumPorts));
      }
   }
   
} /* listall */
