/**
 * @file config.cc
 *
 * Config - Handling of configuration data.
 *
 * Copyright: 2003 Thomas Wintergerst. All rights reserved.
 *
 * $FreeBSD$
 * $Id: config.cc,v 1.10.2.2 2004/12/23 10:41:54 thomas Exp $
 * Project  CAPI for BSD
 * Target   daicctl - Maintenance tool for Diehl legacy active ISDN controllers
 * @date    09.03.2003
 * @author  "Thomas Wintergerst" <twinterg@gmx.de>
 * <p>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * </p>
 */

#include <sys/cdefs.h>
__FBSDID("$FreeBSD$");

// System includes
#include <iostream>

// Import includes

#define __CONFIG__

// Local includes
#ifndef __DAICCTL_H
#  include "daicctl.h"
#endif
#ifndef __CONFIG_H
#  include "config.h"
#endif

using namespace thwutil;





// === Private definitions ===============================================





/// Enumeration list for the Layer2 setting.
static CEnumStringParamCmdOpt::EnumDef_t g_paLayer2SettingsEnum [] =
   {
      { 0, "demand"    },
      { 1, "nodisc"    },
      { 2, "perm"      },
      { 2, "permanent" }
   };

/// Enumeration list for the CRC4 setting.
static CEnumStringParamCmdOpt::EnumDef_t g_paCrc4SettingsEnum [] =
   {
      { 0, "def"     },
      { 0, "default" },
      { 1, "on"      },
      { 2, "off"     }
   };





// === Definitions for public declarations ===============================





/// The global config object.
CConfig e_oConfig;





// === Prototypes for private functions ==================================





// === Implementation of class members ===================================





// --- Class for configuration data handling -----------------------------

/**
 * Constructor.
 */

CConfig::CConfig (void):
   m_oCmdLineEvaluator (),
   // Note: "-l2" must stand before "-l" in order to be recognized.
   m_oCmdOptLayer2 (m_oCmdLineEvaluator, "l2",
                    CCmdOptBase::HAS_PARAMETER |
                       CCmdOptBase::PARAM_SEPARATOR |
                       CCmdOptBase::IGNORE_PARAM_CASE |
                       CCmdOptBase::UNIQUE_OPTION,
                    NULL,
                    g_paLayer2SettingsEnum,
                    ARRAY_COUNT (g_paLayer2SettingsEnum),
                    1),
   m_oCmdOptList (m_oCmdLineEvaluator, "l",
                  CCmdOptBase::TERMINATE_EVALUATION, NULL),
   // Note: "-crc4" must stand before "-c" in order to be recognized.
   m_oCmdOptCrc4 (m_oCmdLineEvaluator, "crc4",
                  CCmdOptBase::HAS_PARAMETER |
                     CCmdOptBase::PARAM_SEPARATOR |
                     CCmdOptBase::IGNORE_PARAM_CASE |
                     CCmdOptBase::UNIQUE_OPTION,
                  NULL,
                  g_paCrc4SettingsEnum,
                  ARRAY_COUNT (g_paCrc4SettingsEnum),
                  0),
   m_oCmdOptCtlr (m_oCmdLineEvaluator, "c",
                  CCmdOptBase::HAS_PARAMETER |
                     CCmdOptBase::UNIQUE_OPTION,
                  NULL, 0, 1, 127),
   m_oCmdOptDirectory (m_oCmdLineEvaluator, "d",
                       CCmdOptBase::HAS_PARAMETER |
                          CCmdOptBase::PARAM_SEPARATOR |
                          CCmdOptBase::UNIQUE_OPTION,
                       (char *) NULL, UINT_MAX, DAIC_DEF_DOWNLOAD_DIR),
   m_oCmdOptDChnProt (m_oCmdLineEvaluator, "prot",
                      CCmdOptBase::HAS_PARAMETER |
                         CCmdOptBase::PARAM_SEPARATOR |
                         CCmdOptBase::PARAM_OPTIONAL |
                         CCmdOptBase::UNIQUE_OPTION,
                      (char *) NULL, UINT_MAX, "etsi"),
   m_oCmdOptP2P (m_oCmdLineEvaluator, "p2p",
                 CCmdOptBase::HAS_PARAMETER |
                    CCmdOptBase::PARAM_SEPARATOR |
                    CCmdOptBase::PARAM_OPTIONAL |
                    CCmdOptBase::UNIQUE_OPTION,
                 NULL, -1, 0, 63),
   m_oCmdOptPermanent (m_oCmdLineEvaluator, "perm", 0, NULL),
   m_oCmdOptLowChannel (m_oCmdLineEvaluator, "w",
                        CCmdOptBase::HAS_PARAMETER |
                           CCmdOptBase::PARAM_SEPARATOR |
                           CCmdOptBase::UNIQUE_OPTION,
                        NULL, 0, 0, 30),
   m_oCmdOptOad1 (m_oCmdLineEvaluator, "oad1",
                  CCmdOptBase::HAS_PARAMETER |
                     CCmdOptBase::PARAM_SEPARATOR |
                     CCmdOptBase::UNIQUE_OPTION,
                  (char *) NULL, 32, ""),
   m_oCmdOptOad2 (m_oCmdLineEvaluator, "oad2",
                  CCmdOptBase::HAS_PARAMETER |
                     CCmdOptBase::PARAM_SEPARATOR |
                     CCmdOptBase::UNIQUE_OPTION,
                  (char *) NULL, 32, ""),
   m_oCmdOptOsa1 (m_oCmdLineEvaluator, "osa1",
                  CCmdOptBase::HAS_PARAMETER |
                     CCmdOptBase::PARAM_SEPARATOR |
                     CCmdOptBase::UNIQUE_OPTION,
                  (char *) NULL, 32, ""),
   m_oCmdOptOsa2 (m_oCmdLineEvaluator, "osa2",
                  CCmdOptBase::HAS_PARAMETER |
                     CCmdOptBase::PARAM_SEPARATOR |
                     CCmdOptBase::UNIQUE_OPTION,
                  (char *) NULL, 32, ""),
   m_oCmdOptSpid1 (m_oCmdLineEvaluator, "spid1",
                   CCmdOptBase::HAS_PARAMETER |
                      CCmdOptBase::PARAM_SEPARATOR |
                      CCmdOptBase::UNIQUE_OPTION,
                   (char *) NULL, 32, ""),
   m_oCmdOptSpid2 (m_oCmdLineEvaluator, "spid2",
                   CCmdOptBase::HAS_PARAMETER |
                      CCmdOptBase::PARAM_SEPARATOR |
                      CCmdOptBase::UNIQUE_OPTION,
                   (char *) NULL, 32, ""),
   m_oCmdOptMemCheck (m_oCmdLineEvaluator, "mem",
                      CCmdOptBase::HAS_PARAMETER |
                         CCmdOptBase::PARAM_SEPARATOR |
                         CCmdOptBase::PARAM_OPTIONAL |
                         CCmdOptBase::IGNORE_PARAM_CASE,
                      NULL, true),
   m_oCmdOptHelp1 (m_oCmdLineEvaluator, "h",
                   CCmdOptBase::TERMINATE_EVALUATION, NULL),
   m_oCmdOptHelp2 (m_oCmdLineEvaluator, "?",
                   CCmdOptBase::TERMINATE_EVALUATION, NULL)
{
} // CConfig::CConfig





/**
 * Destructor.
 */

CConfig::~CConfig (void)
{
} // CConfig::~CConfig





/**
 * Perform general initialization.
 *
 * As this class is designed to create a static object, the constructor
 * will not get enough information for initializing the object. This will
 * thus be done by calling this member function.
 *
 * The main task is to set all internal member variables to reasonable
 * defaults. In addition the short program name is determined from the
 * argument specified. It is expected to be a file name, possibly
 * including some path specification. The basename of this file name will
 * be set as the program name.
 *
 * @param pszProgFileName I: The file name of the current program, maybe
 *                           including a path specification.
 *
 * @retval None.
 *
 * @throws std::invalid_argument
 *                        The argument is the NULL pointer or the empty
 *                        string.
 */

void CConfig::Init
   (const char *pszProgFileName)
{
   const char *p;
   
   p = strrchr (pszProgFileName, '/');
   if (p)
   {
      m_strProgName = std::string (p + 1);
   }
   else
   {
      m_strProgName = std::string (pszProgFileName);
   }
   
} // CConfig::Init





/**
 * Evalutate the command line.
 *
 * The command line is evaluated up to a non-option argument or to the
 * end of the command line. As no such command line argument is supported
 * in this program, the existence of one will lead to the result
 * TASK_ERROR.
 *
 * @param iArgc           I: The number of command line arguments in
 *                           papszArgv.
 * @param papszArgv       I: The command line arguments as an array of
 *                           pointers to character vectors.
 *
 * @retval TASK_UNKNOWN   No main action specified, i.e. neither "-l" nor
 *                        "-c#" was specified on the command line.
 * @retval TASK_ERROR     An error occurred evaluating the command line.
 * @retval TASK_USAGE     Only the help text shall be printed.
 * @retval TASK_LIST      The task is to list all available controllers.
 * @retval TASK_DOWNLOAD  The task is to download bootstrap code and
 *                        firmware to a controller.
 */

CConfig::TaskToExecute_t CConfig::EvalCommandLine
   (int    iArgc,
    char **papszArgv)
{
   CCmdLineEvaluator::CmdLineEvalResult_t resEval;
   int                                    iEndIdx;
   
   // evaluate the command line through the command line evaluator object
   resEval = m_oCmdLineEvaluator.EvalCommandLine
                (iArgc, papszArgv, 1, &iEndIdx);
   if (resEval != CCmdLineEvaluator::CLE_OK &&
       resEval != CCmdLineEvaluator::CLE_EVALUATION_TERMINATED)
   {
      // all values except O.K. and termination are real errors
      std::cerr << m_strProgName << ": Error in command line: "
                << m_oCmdLineEvaluator.ResultMsg (resEval) << std::endl;
      return (TASK_ERROR);
   }
   
   // check if only usage information is required
   if (m_oCmdOptHelp1.WasFound () || m_oCmdOptHelp2.WasFound ())
   {
      return (TASK_USAGE);
   }
   
   // check if only a controller list is requested
   if (m_oCmdOptList.WasFound ())
   {
      return (TASK_LIST);
   }
   
   // check if a download is requested (is the only remaining choice now)
   if (! m_oCmdOptCtlr.WasFound ())
   {
      std::cerr << m_strProgName
                << ": Neither list nor download nor help switch specified"
                << std::endl;
      return (TASK_UNKNOWN);
   }
   
   // if there is still a command line argument remaining, there is an error
   // (all parameters must be switches with arguments)
   if (iEndIdx < iArgc)
   {
      std::cerr << m_strProgName
                << ": Error in command line: Unsupported parameter \""
                << papszArgv [iEndIdx] << "\"" << std::endl;
      return (TASK_ERROR);
   }
   
   // if we reach this point the only remaining choice is to download a
   // controller and there is no error throughout the command line parameters
   return (TASK_DOWNLOAD);
} // CConfig::EvalCommandLine





/**
 * Print hints for using this program.
 */

void CConfig::Usage (void)
{
   std::cout << std::endl;
   std::cout << m_strProgName
             << " - Download firmware and configuration to legacy Diehl active ISDN boards"
             << std::endl;
   std::cout << std::endl;
   std::cout << "Usage:" << std::endl << std::endl;
   std::cout << m_strProgName << " -l" << std::endl;
   std::cout << "   List available controllers" << std::endl;
   std::cout << std::endl;
   std::cout << m_strProgName
             << " -c# [-d:<path>] [-prot[:{etsi|1tr6|...}]] [-p2p[:#]] [-perm]"
             << std::endl
             << "      [-l2:{demand|nodisc|perm|permanent}] [-w:#] [-crc4:{def|default|on|off}]"
             << std::endl
             << "      [-oad{1|2}:<number>] [-osa{1|2}:<number>] [-spid{1|2}:<number>]"
             << std::endl;
   std::cout << "   Perform download operation" << std::endl;
   std::cout << std::endl;
   std::cout << m_strProgName << " {-h | -?}" << std::endl;
   std::cout << "   Print this help text" << std::endl;
   std::cout << std::endl;
   
   std::cout << "Option meanings:" << std::endl << std::endl;
   std::cout << "   -c#\tSpecify the controller number to download or reset."
             << std::endl << std::endl;
   std::cout << "   -d:<path>" << std::endl;
   std::cout << "\tSet the directory for the download files."
             << std::endl << std::endl;
   std::cout << "   -prot[:{etsi|1tr6|...}]" << std::endl;
   std::cout << "\tSpecify the D-channel protocol to download (default \"etsi\"). There"
             << std::endl;
   std::cout << "\tmust be a file with a name matching the switch argument. If no argument"
             << std::endl;
   std::cout << "\tis specified for this switch, the controller is only reset."
             << std::endl << std::endl;
   std::cout << "   -p2p[:#]" << std::endl;
   std::cout << "\tSpecify point-to-point operation (default is multipoint operation). If"
             << std::endl;
   std::cout << "\tanother TEI value than zero should be used, the optional numeric"
             << std::endl;
   std::cout << "\targument must be given."
             << std::endl << std::endl;
   std::cout << "   -perm" << std::endl;
   std::cout << "\tSpecify operation for a permanent connection."
             << std::endl << std::endl;
   std::cout << "   -l2:{demand|nodisc|perm|permanent}" << std::endl;
   std::cout << "\tSpecify Layer 2 operation mode (default is \"nodisc\")."
             << std::endl << std::endl;
   std::cout << "   -w:#\tModify the setting of the low channel value for S2m boards."
             << std::endl << std::endl;
   std::cout << "   -crc4:{def|default|on|off}" << std::endl;
   std::cout << "\tModify the CRC4 setting for S2m boards."
             << std::endl << std::endl;
   std::cout << "   -oad{1|2}:<number>" << std::endl;
   std::cout << "\tSet the originating address value for U.S. protocols."
             << std::endl << std::endl;
   std::cout << "   -osa{1|2}:<number>" << std::endl;
   std::cout << "\tSet the originating subaddress value for U.S. protocols."
             << std::endl << std::endl;
   std::cout << "   -spid{1|2}:<number>" << std::endl;
   std::cout << "\tSet the service profile identifier value for U.S. protocols."
             << std::endl << std::endl;
   std::cout << "   -mem[:y|1|n|0]" << std::endl;
   std::cout << "\tLet the board perform an internal memory check."
             << std::endl;

   std::cout << std::endl;
   
   std::cout << "The download operation automatically determines the file names to use from"
             << std::endl;
   std::cout << "the board type and the D-channel protocol. All files must be stored in the"
             << std::endl;
   std::cout << "download directory (default is \"" << DAIC_DEF_DOWNLOAD_DIR
             << "\", may be changed through"
             << std::endl;
   std::cout << "switch \"-d:<path>\"). Only the \"etsi\" D-channel protocol is available for"
             << std::endl;
   std::cout << "all boards. But if there are download files matching the D-channel protocol"
             << std::endl;
   std::cout << "specified on the command line, other protocols may be used. The last three"
             << std::endl;
   std::cout << "groups of switches are only needed for US D-channel protocols."
             << std::endl;
   std::cout << std::endl;

   exit(1);
} // CConfig::Usage





/**
 * General configuration: Get program name.
 */

const char *CConfig::GetProgName (void)
{
   return (m_strProgName.c_str ());
} // CConfig::GetProgName





/**
 * Query for the current base directory for controller software files.
 */

std::string CConfig::GetBaseDirectory (void)
{
   return (m_oCmdOptDirectory.GetValue ());
} // CConfig::GetBaseDirectory





/**
 * Query for the current controller number.
 */

unsigned CConfig::GetController (void)
{
   return ((unsigned) (m_oCmdOptCtlr.GetValue ()));
} // CConfig::GetController





/**
 * Query for the current D-channel protocol.
 */

std::string CConfig::GetDChannelProtocol (void)
{
   return (m_oCmdOptDChnProt.GetValue ());
} // CConfig::GetDChannelProtocol




/**
 * Query for the current point-to-point or multipoint setting.
 */

bool CConfig::GetPointToPointOperation (void)
{
   return (m_oCmdOptP2P.WasFound ());
} // CConfig::GetPointToPointOperation





/**
 * Query for the current TEI value setting.
 */

int CConfig::GetTeiValue (void)
{
   return (m_oCmdOptP2P.GetValue ());
} // CConfig::GetTeiValue





/**
 * Query for the permanent connection setting.
 */

bool CConfig::GetPermanentConnection (void)
{
   return (m_oCmdOptPermanent.WasFound ());
} // CConfig::GetPermanentConnection





/**
 * Query for the D-channel layer 2 connection setting.
 *
 *
 * The default setting for the layer 2 setting depends on the board type.
 * For S0 board the default is "no-disconnect", for S2m boards it is
 * "permanent". But a specification on the command line overwrites any
 * default value.
 *
 * @param pszBoardName          I: The name of the board for which the setting
 *                                 is to be returned.
 *
 * @retval 0                    On-demand connection.
 * @retval 1                    No local disconnect after established connection.
 * @retval 2                    Permanent connection, build it up every time the
 *                              layer 2 is disconnected by the remote side.
 */

unsigned CConfig::GetLayer2Setting
   (const char *pszBoardName)
{
   if (m_oCmdOptLayer2.WasFound ())
   {
      return ((unsigned) (m_oCmdOptLayer2.GetValue ()));
   }
   if (strcasestr (pszBoardName, "S2m") == 0)
   {
      return (1);
   }
   return (2);
} // CConfig::GetLayer2Setting





/**
 * Query for the low channel setting for S2m boards.
 */

unsigned CConfig::GetLowChannel (void)
{
   return (m_oCmdOptLowChannel.GetValue ());
} // CConfig::GetLowChannel





/**
 * Query for the CRC4 setting for S2m boards.
 */

unsigned CConfig::GetCrc4Setting (void)
{
   return ((unsigned) (m_oCmdOptCrc4.GetValue ()));
} // CConfig::GetCrc4Setting





/**
 * Query for the option to let the board perform a memory check.
 */

bool CConfig::GetDoMemCheck (void)
{
   return (m_oCmdOptMemCheck.WasFound () && m_oCmdOptMemCheck.GetValue ());
} // CConfig::GetDoMemCheck





/**
 * Query for the originator address for U.S. protocols.
 */

std::string CConfig::GetOad
   (int i)
{
   if (i == 1)
   {
      return (m_oCmdOptOad1.GetValue ());
   }
   else
   {
      return (m_oCmdOptOad2.GetValue ());
   }
   
} // CConfig::GetOad





/**
 * Query for the originator subaddress for U.S. protocols.
 */

std::string CConfig::GetOsa
   (int i)
{
   if (i == 1)
   {
      return (m_oCmdOptOsa1.GetValue ());
   }
   else
   {
      return (m_oCmdOptOsa2.GetValue ());
   }
   
} // CConfig::GetOsa





/**
 * Query for the service profile identifier for U.S. protocols.
 */

std::string CConfig::GetSpid
   (int i)
{
   if (i == 1)
   {
      return (m_oCmdOptSpid1.GetValue ());
   }
   else
   {
      return (m_oCmdOptSpid2.GetValue ());
   }
   
} // CConfig::GetSpid





// === Implementation of public functions ================================





// === Implementation of private functions ===============================
