/**
 * @file config.h
 *
 * Config - Handling of configuration data.
 *
 * Copyright: 2003 Thomas Wintergerst. All rights reserved.
 *
 * $FreeBSD$
 * $Id: config.h,v 1.6.2.2 2004/12/23 10:41:54 thomas Exp $
 * Project  CAPI for BSD
 * Target   daicctl - Maintenance tool for Diehl legacy active ISDN controllers
 * @date    09.03.2003
 * @author  "Thomas Wintergerst" <twinterg@gmx.de>
 * <p>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * </p>
 */

#ifndef __CONFIG_H
#define __CONFIG_H

#include <stddef.h> // size_t
#include <list>
#include <string>

#ifndef __CMDOPT2_H
#  include "cmdopt2.h"
#endif





// === Public declarations ===============================================





// === Class declarations ================================================





/**
 * Class for configuration data handling.
 *
 * This class is implemented as a singleton, i.e. there will be exactly one
 * object instance of this class. This object will be defined as a global static
 * variable exported by this module. Thus there is no need for a GetInstance()
 * method like in the general singleton pattern.
 */
class CConfig
{
   public:
      /// An enumeration of the possible tasks of the program.
      enum TaskToExecute_t
      {
         TASK_UNKNOWN,          ///< No task specified on the command line.
         TASK_ERROR,            ///< An error occurred evaluation the command
                                ///< line.
         TASK_USAGE,            ///< The usage hints shall be put out.
         TASK_LIST,             ///< List all available controllers.
         TASK_DOWNLOAD          ///< Download firmware to a controller.
      };
      
      
      
      /**
       * Constructor.
       */
      CConfig (void);
      
      /**
       * Destructor.
       */
      ~CConfig (void);
      
      /**
       * Perform general initialization.
       *
       * As this class is designed to create a static object, the constructor
       * will not get enough information for initializing the object. This will
       * thus be done by calling this member function.
       *
       * The main task is to set all internal member variables to reasonable
       * defaults. In addition the short program name is determined from the
       * argument specified. It is expected to be a file name, possibly
       * including some path specification. The basename of this file name will
       * be set as the program name.
       *
       * @param pszProgFileName I: The file name of the current program, maybe
       *                           including a path specification.
       *
       * @retval None.
       *
       * @throws std::invalid_argument
       *                        The argument is the NULL pointer or the empty
       *                        string.
       */
      void Init
         (const char *pszProgFileName);

      /**
       * Evalutate the command line.
       *
       * The command line is evaluated up to a non-option argument or to the
       * end of the command line. As no such command line argument is supported
       * in this program, the existence of one will lead to the result
       * TASK_ERROR.
       *
       * @param iArgc           I: The number of command line arguments in
       *                           papszArgv.
       * @param papszArgv       I: The command line arguments as an array of
       *                           pointers to character vectors.
       *
       * @retval TASK_UNKNOWN   No main action specified, i.e. neither "-l" nor
       *                        "-d#" was specified on the command line.
       * @retval TASK_ERROR     An error occurred evaluating the command line.
       * @retval TASK_USAGE     Only the help text shall be printed.
       * @retval TASK_LIST      The task is to list all available controllers.
       * @retval TASK_DOWNLOAD  The task is to download bootstrap code and
       *                        firmware to a controller.
       */
      TaskToExecute_t EvalCommandLine
         (int    iArgc,
          char **papszArgv);

      /**
       * Print hints for using this program.
       */
      void Usage (void);

      /**
       * General configuration: Get program name.
       */
      const char *GetProgName (void);
      
      /**
       * Query for the current base directory for controller software files.
       */
      std::string GetBaseDirectory (void);
      
      /**
       * Query for the current controller number.
       */
      unsigned GetController (void);
      
      /**
       * Query for the current D-channel protocol.
       */
      std::string GetDChannelProtocol (void);
      
      /**
       * Query for the current point-to-point or multipoint setting.
       */
      bool GetPointToPointOperation (void);
      
      /**
       * Query for the current TEI value setting.
       */
      int GetTeiValue (void);
      
      /**
       * Query for the permanent connection setting.
       */
      bool GetPermanentConnection (void);
      
      /**
       * Query for the D-channel layer 2 connection setting.
       *
       * The default setting for the layer 2 setting depends on the board type.
       * For S0 board the default is "no-disconnect", for S2m boards it is
       * "permanent". But a specification on the command line overwrites any
       * default value.
       *
       * @param pszBoardName    I: The name of the board for which the setting
       *                           is to be returned.
       *
       * @retval 0              On-demand connection.
       * @retval 1              No local disconnect after established connection.
       * @retval 2              Permanent connection, build it up every time the
       *                        layer 2 is disconnected by the remote side.
       */
      unsigned GetLayer2Setting
         (const char *pszBoardName);
      
      /**
       * Query for the low channel setting for S2m boards.
       */
      unsigned GetLowChannel (void);
      
      /**
       * Query for the CRC4 setting for S2m boards.
       */
      unsigned GetCrc4Setting (void);
      
      /**
       * Query for the option to let the board perform a memory check.
       */
      bool GetDoMemCheck (void);
      
      /**
       * Query for the originator address for U.S. protocols.
       */
      std::string GetOad
         (int i);
      
      /**
       * Query for the originator subaddress for U.S. protocols.
       */
      std::string GetOsa
         (int i);
      
      /**
       * Query for the service profile identifier for U.S. protocols.
       */
      std::string GetSpid
         (int i);
      


   protected:



   private:
      /// The short program file name.
      std::string m_strProgName;
      
      /// The command line evaluator object.
      thwutil::CCmdLineEvaluator m_oCmdLineEvaluator;
      
      /// The option object for the layer 2 setting (-l2:demand|nodisc|perm).
      thwutil::CEnumStringParamCmdOpt m_oCmdOptLayer2;
      
      /// The option object for the task of listing all controllers (-l).
      thwutil::CBooleanCmdOpt m_oCmdOptList;
      
      /// The option object for the crc4 setting of S2m boards
      /// (-crc4:def|on|off).
      thwutil::CEnumStringParamCmdOpt m_oCmdOptCrc4;
      
      /// The option object for the controller number to download (-c#).
      thwutil::CUIntParamCmdOpt m_oCmdOptCtlr;
      
      /// The option object for the directory for all download files (-d:<path>).
      thwutil::CStringParamCmdOpt m_oCmdOptDirectory;
      
      /// The option object for the D-channel protocol (-prot:{etsi|1tr6|...}).
      thwutil::CStringParamCmdOpt m_oCmdOptDChnProt;
      
      /// The option object for P2P configuration (-p2p[:#]).
      thwutil::CIntParamCmdOpt m_oCmdOptP2P;
      
      /// The option object for permanent line connections (-perm).
      thwutil::CBooleanCmdOpt m_oCmdOptPermanent;
      
      /// The option object for the low channel setting of S2m boards (-w:#).
      thwutil::CUIntParamCmdOpt m_oCmdOptLowChannel;
      
      /// The option object for the originator address of the first B-channel of
      /// S0 boards for north american D-channel protocols (-oad1:<number>).
      thwutil::CStringParamCmdOpt m_oCmdOptOad1;
      
      /// The option object for the originator address of the second B-channel
      /// of S0 boards for north american D-channel protocols (-oad2:<number>).
      thwutil::CStringParamCmdOpt m_oCmdOptOad2;
      
      /// The option object for the originator subaddress of the first B-channel
      /// of S0 boards for north american D-channel protocols (-osa1:<number>).
      thwutil::CStringParamCmdOpt m_oCmdOptOsa1;
      
      /// The option object for the originator subaddress of the second
      /// B-channel of S0 boards for north american D-channel protocols
      /// (-osa2:<number>).
      thwutil::CStringParamCmdOpt m_oCmdOptOsa2;
      
      /// The option object for the SPID of the first B-channel of S0 boards for
      /// north american D-channel protocols (-spid1:<number>).
      thwutil::CStringParamCmdOpt m_oCmdOptSpid1;
      
      /// The option object for the SPID of the second B-channel of S0 boards
      /// for north american D-channel protocols (-spid2:<number>).
      thwutil::CStringParamCmdOpt m_oCmdOptSpid2;
      
      /// The option object to perform memory check or not (-mem[:j|y|1|n|0]).
      thwutil::CBooleanCmdOpt m_oCmdOptMemCheck;

      /// The option object for help output by "-h".
      thwutil::CBooleanCmdOpt m_oCmdOptHelp1;
      
      /// The option object for help output by "-?".
      thwutil::CBooleanCmdOpt m_oCmdOptHelp2;



}; // CConfig





/// The global config object.
extern CConfig e_oConfig;





// === Prototypes of interface functions =================================





#endif // __CONFIG_H
