/**
 * @file daicctl.cc
 *
 * DaicCtl - Argument parsing and execution of operations.
 *
 * Copyright: 2003 Thomas Wintergerst. All rights reserved.
 *
 * $FreeBSD$
 * $Id: daicctl.cc,v 1.8.2.2 2004/12/23 10:41:54 thomas Exp $
 * Project  CAPI for BSD
 * Target   daicctl - Maintenance tool for Diehl legacy active ISDN controllers
 * @date    09.03.2000
 * @author  "Thomas Wintergerst" <twinterg@gmx.de>
 * <p>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * </p>
 */

#include <sys/cdefs.h>
__FBSDID("$FreeBSD$");

/* system includes */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/fcntl.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/mman.h>
#include <capi20.h>
#include <capi_bsd.h>
#include <string>
#include <iostream>
#include <iomanip>
#include <errno.h>

/* import includes */

#define __DAICCTL__

/* local includes */
#include "config.h"
#include "daicctl.h"





/* === public definitions ================================================ */





/** The name of the program for output. */
const char *e_pszProgName = NULL;





/* === private definitions =============================================== */





/* === prototypes for private functions ================================== */





/**
 * List all controllers installed.
 */
static int ListControllers (void);

/**
 * Perform download operation.
 */
static int Download (void);





/* === definition of public functions ==================================== */





/**
 * The main entry point.
 */

int main
   (int    iArgc,
    char **papszArgv)
{
   CConfig::TaskToExecute_t task;
   
   /* initialize the configuration object */
   e_oConfig.Init (papszArgv [0]);
   
   /* parse the command line */
   task = e_oConfig.EvalCommandLine (iArgc, papszArgv);

   /* distinguish between the possible task values */
   switch (task)
   {
      case CConfig::TASK_ERROR:
      default:
      case CConfig::TASK_UNKNOWN:
      case CConfig::TASK_USAGE:
         e_oConfig.Usage ();
         return (1);

      case CConfig::TASK_LIST:
         return (ListControllers ());

      case CConfig::TASK_DOWNLOAD:
         return (Download ());
   }
   
} // main





/* === definition of private functions =================================== */





/**
 * List all controllers installed.
 */

static int ListControllers (void)
{
   CAPIProfileBuffer_t  profile;
   CAPICtlrDriverInfo_t drvInfo;
   int                  i;
   unsigned             uNumCtlr;
   unsigned             u;
   unsigned             uRes;
   
   /* first get the number of installed controllers */
   uRes = capi20_get_profile (0, &profile);
   if (uRes != CAPI_OK)
   {
      fprintf (stderr,
               "%s: error getting number of installed controllers: 0x%04x\n",
               e_pszProgName, uRes);
      return (2);
   }
   uNumCtlr = C_GET_WORD (profile.wCtlr);
   if (uNumCtlr == 0)
   {
      printf ("%s: no controllers installed\n", e_pszProgName);
      return (2);
   }
   
   /* now query all controllers until all found */
   printf ("\n%u controllers installed:\n\n", uNumCtlr);
   for (i = 1, u = 0; i <= 127 && u < uNumCtlr; i++)
   {
      uRes = capi20_get_ctlr_driver_info ((unsigned) i, &drvInfo);
      if (uRes == CRE_CAPI_NOT_INSTALLED)
      {
         /* seems to be a hole in the array of CAPI controllers... */
         continue;
      }
      u++;                              /* count the controller found */
      if (uRes != CAPI_OK)
      {
         fprintf (stderr,
                  "%s: error getting driver info for controller %u: 0x%04x\n",
                  e_pszProgName, (unsigned) i, uRes);
         continue;
      }
      
      printf ("\t%u: name \"%s\", type \"%s\", driver \"%s\", driver unit "
                 "%u\n",
              (unsigned) i, drvInfo.szCtlrName, drvInfo.szCtlrTypeName,
              drvInfo.szDriverName, C_GET_WORD (drvInfo.wDrvUnitNum));
      if (C_GET_WORD (drvInfo.wNumPorts > 1))
      {
         printf ("\t\tport %u of %u\n",
                 C_GET_WORD (drvInfo.wPortIdx),
                 C_GET_WORD (drvInfo.wNumPorts));
      }
   }
   
   return (0);
} // ListControllers





/**
 * Perform download operation.
 */

static int Download (void)
{
   unsigned              uCtlr;
   CAPICtlrDriverInfo_t  capiDrvInfo;
   std::string           strDChnProt;
   std::string           strBaseDir;
   std::string           strBootCodeFileName;
   std::string           strFirmwareFileName;
   int                   fdBootCode;
   int                   fdFirmware;
   struct stat           statbuf;
   size_t                nLenBootCode;
   size_t                nLenFirmware;
   DaicBoardConfigData_t configData;
   CAPICtlrDataBlock_t   aCtlrDataBlocks [3];
   int                   iTeiVal;
   unsigned              uRes;

   // first get driver information about the controller specified
   uCtlr = e_oConfig.GetController ();
   uRes = capi20_get_ctlr_driver_info (uCtlr, &capiDrvInfo);
   if (uRes != CAPI_OK)
   {
      std::cerr << e_oConfig.GetProgName ()
                << ": Error getting driver info about controller " << uCtlr
                << ": 0x" << std::setfill ('0') << std::setw (4)
                << std::setbase (16) << uRes << std::endl;
      return (1);
   }
   
   // check if this is really a Diehl legacy active ISDN controller
   if (strcasecmp (capiDrvInfo.szDriverName, DAIC_DRIVER_NAME) != 0)
   {
      std::cerr << e_oConfig.GetProgName ()
                << ": Controller " << uCtlr
                << " is no Diehl legacy active ISDN controller"
                << std::endl;
      return (1);
   }

   // download may only be performed for the first port of a multi port board
   if (C_GET_WORD (capiDrvInfo.wPortIdx) != 0)
   {
      std::cerr << e_oConfig.GetProgName ()
                << ": Controller " << uCtlr
                << " is not the first port of its board" << std::endl;
      return (1);
   }
   
   // the controller specified is valid for download, distinguish between reset
   // or download operation
   strDChnProt = e_oConfig.GetDChannelProtocol ();
   if (strDChnProt.length () == 0)
   {
      // no D-channel protocol --> only reset operation
      uRes = capi20_reset_ctlr (uCtlr, 0, NULL);
      if (uRes != CAPI_OK)
      {
         std::cerr << e_oConfig.GetProgName ()
                   << ": Error unloading controller " << uCtlr << ": 0x"
                   << std::setfill ('0') << std::setw (4) << std::setbase (16)
                   << uRes << std::endl;
         return (1);
      }
      std::cout << e_oConfig.GetProgName ()
                << ": Controller " << uCtlr << " successfully unloaded"
                << std::endl;
      return (0);
   }
   
   // real download operation requested; collect all necessary files and the
   // data for the configuration block

   // all files (boot code and protocol firmware) must lie under the same
   // directory
   strBaseDir = e_oConfig.GetBaseDirectory ();
   // be sure the directory name ends with a path separator character
   if (strBaseDir.length () <= 0 ||
       strBaseDir [strBaseDir.length () - 1] != '/')
   {
      strBaseDir += '/';
   }
   
   // open the file for the boot code (an S2m board needs a different boot code
   // than an S0 board)
   if (strcasestr (capiDrvInfo.szCtlrTypeName, "S2m") == NULL)
   {
      strBootCodeFileName = strBaseDir + "dnload.bin";
   }
   else
   {
      strBootCodeFileName = strBaseDir + "prload.bin";
   }
   fdBootCode = open (strBootCodeFileName.c_str (), O_RDONLY);
   if (fdBootCode == -1)
   {
      std::cerr << e_oConfig.GetProgName ()
                << ": Error opening boot code file \"" << strBootCodeFileName
                << "\": " << strerror (errno) << " (" << errno << ")"
                << std::endl;
      return (1);
   }
   if (fstat (fdBootCode, &statbuf) != 0)
   {
      std::cerr << e_oConfig.GetProgName ()
                << ": Error getting size of file \"" << strBootCodeFileName
                << "\": " << strerror (errno) << " (" << errno << ")"
                << std::endl;
      (void) close (fdBootCode);
      return (1);
   }
   nLenBootCode = (size_t) (statbuf.st_size);
   
   // open the file for the protocol firmware (must distinguish between the
   // different board types
   if (strcasestr (capiDrvInfo.szCtlrTypeName, "S2m") != NULL)
   {
      strFirmwareFileName = strBaseDir + "te_" + strDChnProt + ".p";
   }
   else if (strcasestr (capiDrvInfo.szCtlrTypeName, "Quadro") != NULL)
   {
      strFirmwareFileName = strBaseDir + "te_" + strDChnProt + ".sq";
   }
   else if (strcasestr (capiDrvInfo.szCtlrTypeName, "SCOM") != NULL)
   {
      strFirmwareFileName = strBaseDir + "te_" + strDChnProt + ".sy";
   }
   else if (strcasestr (capiDrvInfo.szCtlrTypeName, "Sx") != NULL)
   {
      strFirmwareFileName = strBaseDir + "te_" + strDChnProt + ".sx";
   }
   else
   {
      strFirmwareFileName = strBaseDir + "te_" + strDChnProt + ".bin";
   }
   fdFirmware = open (strFirmwareFileName.c_str (), O_RDONLY);
   if (fdFirmware == -1)
   {
      std::cerr << e_oConfig.GetProgName ()
                << ": Error opening firmware file \""
                << strFirmwareFileName << "\": "
                << strerror (errno) << " (" << errno << ")" << std::endl;
      (void) close (fdBootCode);
      return (1);
   }
   if (fstat (fdFirmware, &statbuf) != 0)
   {
      std::cerr << e_oConfig.GetProgName ()
                << ": Error getting size of file \"" << strFirmwareFileName
                << "\": " << strerror (errno) << " (" << errno << ")"
                << std::endl;
      (void) close (fdFirmware);
      (void) close (fdBootCode);
      return (1);
   }
   nLenFirmware = (size_t) (statbuf.st_size);
   
   // create a data block for the board configuration
   memset (&configData, 0, sizeof (configData));
   iTeiVal = e_oConfig.GetTeiValue ();
   if (iTeiVal < 0)
   {
      configData.ucTei = 0;
   }
   else
   {
      configData.ucTei = (unsigned char) ((iTeiVal << 1) | 0x01);
   }
   configData.ucNt2 = (unsigned char) e_oConfig.GetPointToPointOperation ();
   configData.ucPermanent = (unsigned char) e_oConfig.GetPermanentConnection ();
   configData.ucStableL2 =
      (unsigned char) e_oConfig.GetLayer2Setting (capiDrvInfo.szCtlrTypeName);
   configData.ucLowChannel = (unsigned char) e_oConfig.GetLowChannel ();
   configData.ucCrc4 = (unsigned char) e_oConfig.GetCrc4Setting ();
   e_oConfig.GetOad (1).copy ((char *) configData.aucOad1,
                              sizeof (configData.aucOad1) - 1);
   e_oConfig.GetOsa (1).copy ((char *) configData.aucOsa1,
                              sizeof (configData.aucOsa1) - 1);
   e_oConfig.GetSpid (1).copy ((char *) configData.aucSpid1,
                               sizeof (configData.aucSpid1) - 1);
   e_oConfig.GetOad (2).copy ((char *) configData.aucOad2,
                              sizeof (configData.aucOad2) - 1);
   e_oConfig.GetOsa (2).copy ((char *) configData.aucOsa2,
                              sizeof (configData.aucOsa2) - 1);
   e_oConfig.GetSpid (2).copy ((char *) configData.aucSpid2,
                               sizeof (configData.aucSpid2) - 1);
   configData.ucMemBootOpt = (unsigned char) e_oConfig.GetDoMemCheck ();
   
   // prepare the arguments for the download request
   bzero (aCtlrDataBlocks, sizeof (aCtlrDataBlocks));
   aCtlrDataBlocks [0].nLenDataBlock = nLenBootCode;
   aCtlrDataBlocks [0].paucDataBlock =
      (unsigned char *)
         mmap (NULL, nLenBootCode, PROT_READ, MAP_PRIVATE, fdBootCode, 0);
   if (aCtlrDataBlocks [0].paucDataBlock == MAP_FAILED)
   {
      std::cerr << e_oConfig.GetProgName ()
                << ": Error mapping boot code file content into memory: "
                << strerror (errno) << " (" << errno << ")" << std::endl;
      (void) close (fdFirmware);
      (void) close (fdBootCode);
      return (1);
   }
   aCtlrDataBlocks [1].nLenDataBlock = nLenFirmware;
   aCtlrDataBlocks [1].paucDataBlock =
      (unsigned char *)
         mmap (NULL, nLenFirmware, PROT_READ, MAP_PRIVATE, fdFirmware, 0);
   if (aCtlrDataBlocks [1].paucDataBlock == MAP_FAILED)
   {
      std::cerr << e_oConfig.GetProgName ()
                << ": Error mapping boot firmware file content into memory: "
                << strerror (errno) << " (" << errno << ")" << std::endl;
      (void) munmap (aCtlrDataBlocks [0].paucDataBlock,
                     aCtlrDataBlocks [0].nLenDataBlock);
      (void) close (fdFirmware);
      (void) close (fdBootCode);
      return (1);
   }
   aCtlrDataBlocks [2].nLenDataBlock = sizeof (configData);
   aCtlrDataBlocks [2].paucDataBlock = (unsigned char *) &configData;

   // now finally perform the download request
   uRes = capi20_reset_ctlr (uCtlr,
                             ARRAY_COUNT (aCtlrDataBlocks),
                             aCtlrDataBlocks);
   (void) munmap (aCtlrDataBlocks [1].paucDataBlock, 
                  aCtlrDataBlocks [1].nLenDataBlock);
   (void) close (fdFirmware);
   (void) munmap (aCtlrDataBlocks [0].paucDataBlock, 
                  aCtlrDataBlocks [0].nLenDataBlock);
   (void) close (fdBootCode);
   if (uRes != CAPI_OK)
   {
      std::cerr << e_oConfig.GetProgName ()
                << ": Error downloading firmware to controller " << uCtlr
                << ": 0x" << std::setfill ('0') << std::setw (4)
                << std::setbase (16) << uRes << std::endl;
      return (2);
   }

   std::cout << e_oConfig.GetProgName ()
             << ": Downloading firmware to controller " << uCtlr
             << " successful" << std::endl;

   return (0);
} // Download
